<?php
/*
YaotGui Lists


This file is part of YaotGui.

YaotGui is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

YaotGui is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with YaotGui.  If not, see <http://www.gnu.org/licenses/>.

Copyright 2008 Thomas Axelsson

*/

/// A simple list control
class LibList extends LibControl
{
   protected $_itemClass;

   protected $_items = array();
   protected $_selIndex = 0;

   /// Creates a new List
   /** \param $id The id in the HTML code
       \param $class The CSS class. Items will have classes <i>$class</i>_item and <i>$class</i>_item_sel
    */
   function __construct($id, $class)
   {
      parent::__construct($id, $class);
      $this->_itemClass = $class . '_item';
      $this->_type = 'LibList';
   }

   protected function printValues()
   {
      echo "
var __{$this->_id}_selectedIndex = $this->_selIndex;
var __{$this->_id}_values = new Array();\n";
      $i = 0;
      foreach($this->_items as $item)
      {
	 $values = $item->getValues();
	 echo "__{$this->_id}_values[$i] = new Array();\n";
	 foreach($values as $name=>$value)
	 {
	    echo "__{$this->_id}_values[$i].$name = '" . urlencode($value) . "';\n";
	 }
	 $i++;
      }

      echo "
function __{$this->_id}_getSelectedValue(name)
{
	return __{$this->_id}_getValue(__{$this->_id}_selectedIndex, name);
}

function __{$this->_id}_getValue(idx, name)
{
	return urlDecode(__{$this->_id}_values[idx][name]);
}";
   }

   protected function printListScript()
   {
      parent::printScriptStart();
      $this->printValues();
      echo "
function {$this->_jsPrefix}move(step, wrap)
{
	var items = $this->_element.childNodes;
	items[{$this->_jsPrefix}selectedIndex].className = '" . LibListItem::getClassStringByName($this->_itemClass) . "';

	{$this->_jsPrefix}selectedIndex = lib_control_itemStep({$this->_jsPrefix}selectedIndex, step, items.length, wrap);
	items[{$this->_jsPrefix}selectedIndex].className = '" . LibListItem::getSelClassStringByName($this->_itemClass) . "';
	lib_control_focusObject(items[{$this->_jsPrefix}selectedIndex], {$this->_element});

}";
      parent::printScriptEnd();
   }
   
   function printHtml()
   {
      $this->printListScript();
      
      parent::printIdClassDivStart();
      foreach($this->_items as $item)
      {
	 $item->printHtml();
      }
      parent::printDivEnd();
   }

/*   function printDefaultStyle()
   {
      echo ".$this->_class {overflow: hidden;}
";
      LibListItem::printDefaultStyle($this->_itemClass);
      }*/

   function addItem(array $values, $text)
   {
      $this->_items[] = new LibListItem($this->_id . '_' . count($this->_items), $this->_itemClass, $values, $text, count($this->_items) == $this->_selIndex);
   }

   function sort()
   {
      usort($this->_items, array("LibListItem", "LibListItem::compare"));
   }

   public static function getDefaultClass()
   {
      return parent::getDefaultClassByType(get_class());
   }
}

/// List item used in LibList
class LibListItem extends LibControl
{
   protected $_values;
   protected $_text;
   protected $_selected;

   function __construct($id, $class, $values, $text, $selected = false)
   {
      parent::__construct($id, $class);
      $this->_values = $values;
      $this->setText($text);
      $this->setSelected($selected);
      $this->_type = 'LibListItem';
   }

   function printHtml()
   {
      parent::printIdDivStart('class="' . ($this->getSelected() ? $this->getSelClassString() : $this->getClassString()) . '"');
      echo htmlspecialchars($this->_text);
//      parent::printHiddenValues($this->_values);
      parent::printDivEnd();
   }

/*   static function printDefaultStyle($class)
   {
      echo ".$class, ." . self::getSelClassByName($class) ." {padding: 3px; border: 1px solid #fff; font-size: 18px;}
.$class {background: #000;}
.{$class}_sel {background: #555;}
";
}*/

   static function compare(&$item1, &$item2)
   {
      return strcoll($item1->getText(), $item2->getText());
   }

   //accessors
   function getSelClass()
   {
      return self::getSelClassByName($this->_class);
   }

   static function getSelClassByName($class)
   {
       return $class . '_sel';
   }

   function getSelClassString()
   {
      return $this->getSelClassStringByName($this->_class);
   }

   static function getSelClassStringByName($class)
   {
      return parent::getClassStringByNames(self::getSelClassByName(self::getDefaultClass()), self::getSelClassByName($class));
   }

   static function getClassStringByName($class)
   {
      return parent::getClassStringByNames(self::getDefaultClass(), $class);
   }
   
   function setValue($name, $value)
   {
      $this->_values[$name] = $value;
   }

   function getValue($name)
   {
      return $this->_values[$name];
   }

   function getValues()
   {
      return $this->_values;
   }

   function setText($text)
   {
      $this->_text = $text;
   }

   function getText()
   {
      return $this->_text;
   }

   function setSelected($value)
   {
      $this->_selected = $value;
   }

   function getSelected()
   {
      return $this->_selected;
   }

   public static function getDefaultClass()
   {
      return parent::getDefaultClassByType(get_class());
   }
}

/// A list control which output images in tables
class LibImageList extends LibList
{
   protected $_columnCount, $_imgWidth, $_imgHeight;

   /// Creates a new ImageList
   /** \param $id The id in the HTML code
       \param $class The CSS class. Items will have classes <i>$class</i>_item, <i>$class</i>_item_sel and <i>$class</i>_item_img. The table will have the class <i>$class</i>_table.
       \param $columnCount Specifies the number of columns
       \param $imgWidth The width of the images
       \param $imgHeight The height of the images
    */
   function __construct($id, $class, $columnCount, $imgWidth, $imgHeight)
   {
      parent::__construct($id, $class);
      $this->_columnCount = $columnCount;
      $this->_imgWidth = $imgWidth;
      $this->_imgHeight = $imgHeight;
      $this->_type = 'LibImageList';
   }

   function addItem(array $values, $text, $imgSrc, $parse = true)
   {
      $this->_items[] = new LibImageListItem($this->_id . '_' . count($this->_items), $this->_itemClass, $values, $text, $imgSrc, $this->_imgWidth, $this->_imgHeight, count($this->_items) == $this->_selIndex, $parse);
   }

   protected function printListScript()
   {
      parent::printScriptStart();
      $this->printValues();
echo "
var {$this->_jsPrefix}itemCount = " . count($this->_items) . ";
var {$this->_jsPrefix}colCount = $this->_columnCount;
var {$this->_jsPrefix}rowCount = " . ceil(count($this->_items) / $this->_columnCount) . ";

function {$this->_jsPrefix}move(rowStep, colStep, wrap)
{
	var selRow = Math.floor({$this->_jsPrefix}selectedIndex / {$this->_jsPrefix}colCount);
	var selCol = {$this->_jsPrefix}selectedIndex % {$this->_jsPrefix}colCount;
	{$this->_jsPrefix}select(selRow, selCol, false);

	selRow = lib_control_itemStep(selRow, rowStep, {$this->_jsPrefix}rowCount, wrap);
	var maxCol = {$this->_jsPrefix}colCount;
	if(selRow == {$this->_jsPrefix}rowCount - 1)
		maxCol = {$this->_jsPrefix}itemCount - selRow * {$this->_jsPrefix}colCount;
	selCol = lib_control_itemStep(selCol, colStep, maxCol, wrap);
	{$this->_jsPrefix}selectedIndex = selRow * {$this->_jsPrefix}colCount + selCol;

	{$this->_jsPrefix}select(selRow, selCol, true);

}

function {$this->_jsPrefix}move_idx(step, wrap)
{
	var selRow = Math.floor({$this->_jsPrefix}selectedIndex / {$this->_jsPrefix}colCount);
	var selCol = {$this->_jsPrefix}selectedIndex % {$this->_jsPrefix}colCount;
	{$this->_jsPrefix}select(selRow, selCol, false);
	{$this->_jsPrefix}selectedIndex = lib_control_itemStep({$this->_jsPrefix}selectedIndex, step, {$this->_jsPrefix}itemCount, wrap);
	selRow = Math.floor({$this->_jsPrefix}selectedIndex / {$this->_jsPrefix}colCount);
	selCol = {$this->_jsPrefix}selectedIndex % {$this->_jsPrefix}colCount;
	{$this->_jsPrefix}select(selRow, selCol, true);

}

function {$this->_jsPrefix}select(rowNr, colNr, select)
{
	var elem = $this->_element;
	var table = elem.childNodes[0];
	var rows = table.childNodes[0].childNodes;
	var row = rows[rowNr];
	var cols = row.childNodes;
        var col = cols[colNr];
	col.className = select ? '" . LibImageListItem::getSelClassStringByName($this->_itemClass) . "' : '" . LibImageListItem::getClassStringByName($this->_itemClass) . "';
	if(select)
		lib_control_focusObject(col, elem);
}
";
	parent::printScriptEnd();
   }

   function printHtml()
   {
      $this->printListScript();
	 
      parent::printIdClassDivStart();
      parent::printTableStart('class="' . parent::getClassStringByNames($this->getDefaultClass(), $this->getClass(), 'table') . '"');
      parent::printRowStart();
      for($i = 0; $i < count($this->_items); $i++)
      {
	 if($i != 0 && $i % $this->_columnCount == 0)
	 {
	    parent::printRowEnd();
	    parent::printRowStart();
	 }
	 $this->_items[$i]->printHtml();
      }
      parent::printRowEnd();
      parent::printTableEnd();
      parent::printDivEnd();
   }

/*   function printDefaultStyle()
   {
      echo ".$this->_class {overflow: hidden;}
.{$this->_class}_table {table-layout: fixed; width: 100%;}
";
      LibImageListItem::printDefaultStyle($this->_itemClass, $this->_imgWidth, $this->_imgHeight);
      }*/

   public static function getDefaultClass()
   {
      return parent::getDefaultClassByType(get_class());
   }
}

/// Image list item used in LibImageList
class LibImageListItem extends LibListItem
{
   protected $_imgSrc, $_imgWidth, $_imgHeight, $_parse;

   function __construct($id, $class, $values, $text, $imgSrc, $imgWidth, $imgHeight, $selected = false, $parse = true)
   {
      parent::__construct($id, $class, $values, $text, $selected);
      $this->setImgSrc($imgSrc);
      $this->_imgWidth = $imgWidth;
      $this->_imgHeight = $imgHeight;
      $this->_type = 'LibImageListItem';
      $this->_parse = $parse;
   }

   function printHtml()
   {
      parent::printIdColumnStart('class="' . ($this->getSelected() ? $this->getSelClassString() : $this->getClassString()) . '"');
      echo '<img class="' . parent::getClassStringByNames($this->getDefaultClass(), $this->getClass(), 'img') . '" src="' . ($this->_parse ? "getImg.php?src=" . urlencode($this->_imgSrc) . "&keep_ratio=0&width={$this->_imgWidth}&height={$this->_imgHeight}" : $this->_imgSrc) . '">';
      echo htmlspecialchars($this->_text);
      parent::printColumnEnd();
   }

/*   static function printDefaultStyle($class, $imgWidth, $imgHeight)
   {
      echo ".$class, ." . self::getSelClassByName($class) . " {padding: 3px; border: 1px solid #fff; width: " . ($imgWidth + 8) ."px; overflow: hidden; text-align: center; white-space: nowrap; font-size: 18px;}
.$class {background: #000;}
.{$class}_sel {background: #555;}
.{$class}_img {width: {$imgWidth}px; height: {$imgHeight}px; display: block; margin-left: auto; margin-right: auto;}
";
}*/

   //accessors
   function setImgSrc($imgSrc)
   {
      $this->_imgSrc = $imgSrc;
   }

   function getImgSrc()
   {
      return $this->_imgSrc;
   }

   public static function getDefaultClass()
   {
      return parent::getDefaultClassByType(get_class());
   }

   // need to override this because we use 'self'
   static function getSelClassStringByName($class)
   {
      return parent::getClassStringByNames(self::getSelClassByName(self::getDefaultClass()), self::getSelClassByName($class));
   }

   // and this too
   static function getClassStringByName($class)
   {
      return parent::getClassStringByNames(self::getDefaultClass(), $class);
   }
}
?>
